@@ -34,6 +34,7 @@
 //! ```
 use super::{finish_reader, ArrowReader, ArrowResult, RecordBatch};
 use crate::prelude::*;
+use crate::utils::to_arrow_compatible_df;
 use arrow::ipc::{
     reader::FileReader as ArrowIPCFileReader, writer::FileWriter as ArrowIPCFileWriter,
 };
@@ -117,22 +118,23 @@ where
 /// }
 ///
 /// ```
-pub struct IpcWriter<'a, W> {
-    writer: &'a mut W,
+pub struct IpcWriter<W> {
+    writer: W,
 }
 
-impl<'a, W> SerWriter<'a, W> for IpcWriter<'a, W>
+impl<W> SerWriter<W> for IpcWriter<W>
 where
     W: Write,
 {
-    fn new(writer: &'a mut W) -> Self {
+    fn new(writer: W) -> Self {
         IpcWriter { writer }
     }
 
-    fn finish(self, df: &mut DataFrame) -> Result<()> {
+    fn finish(self, df: &DataFrame) -> Result<()> {
+        let df = to_arrow_compatible_df(df);
         let mut ipc_writer = ArrowIPCFileWriter::try_new(self.writer, &df.schema().to_arrow())?;
 
-        let iter = df.iter_record_batches(df.height());
+        let iter = df.iter_record_batches();
 
         for batch in iter {
             ipc_writer.write(&batch)?
